/*
 * Copyright (c) 2007, Ryo Ichinose
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright 
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of the auther nor the 
 *      names of its contributors may be used to endorse or promote products 
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/sleep.h>

#include "timer1.h"
#include "util.h"


#define ICR1_AS_TOP     19999
#define OCR1A_LIMIT_CCW 2100
#define OCR1A_CENTER    1500
#define OCR1A_LIMIT_CW  900

static void timer1_ic_interrupt(void);


int
main(void)
{
	DDRD |= _BV(DDD5);  /* PD5 ԥϤ */
	pullup();

	timer1_ic_enable();
	timer1_ic_set_interrupt(timer1_ic_interrupt);
	timer1_set_wave_mode(WGM1_FAST_PWM_ICR1);
	timer1_set_coma_mode(COM_A_HIGH_FIRST);
	timer1_set_count(0);
	timer1_set_icr1(ICR1_AS_TOP);
	timer1_set_ocr1a(OCR1A_LIMIT_CCW);

	sei();

	/* (8MHz/8)/20000 = 50Hz */
	timer1_start(TIMER1_CK_8);
	
	while (1){
		sleep_mode();
	}

	return 0; /* ٹ򤱤뤿 */
}


static void 
timer1_ic_interrupt(void)
{
	static int ocra[3] = {
		OCR1A_LIMIT_CCW,
		OCR1A_CENTER,
		OCR1A_LIMIT_CW
	};

	static int dir   = 0;
	static int count = 0;

	/* 1Ѥ */
	if (++ count == 50) {
		dir = (dir + 1) % 3;
		count = 0;
		timer1_set_ocr1a(ocra[dir]);
	}
}

/* End of main.c */
