/*
 * Copyright (c) 2007, Ryo Ichinose
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright 
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of the auther nor the 
 *      names of its contributors may be used to endorse or promote products 
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ___TIMER1_H_INCLUDED_
#define ___TIMER1_H_INCLUDED_

#include <avr/io.h>
#include <inttypes.h>


/* ޡåѥޥ */
#define TIMER1_STOP      0x00
#define TIMER1_CK        (_BV(CS10))
#define TIMER1_CK_8      (_BV(CS11))
#define TIMER1_CK_64     (_BV(CS11) | _BV(CS10))
#define TIMER1_CK_256    (_BV(CS12))
#define TIMER1_CK_1024   (_BV(CS12) | _BV(CS10))

/* ӽϥ⡼ѥޥ */
#define COM_A_NONE       0x00
#define COM_A_TOGGLE     (_BV(COM1A0))
#define COM_A_HIGH_FIRST (_BV(COM1A1))
#define COM_A_LOW_FIRST  (_BV(COM1A1) | _BV(COM1A0))

#define COM_B_NONE       0x00
#define COM_B_TOGGLE     (_BV(COM1B0))
#define COM_B_HIGH_FIRST (_BV(COM1B1))
#define COM_B_LOW_FIRST  (_BV(COM1B1) | _BV(COM1B0))

/* ȷѥޥ */
#define WGM1_NORMAL         0x00
#define WGM1_CTC_ICR1       (_BV(WGM13) | _BV(WGM12))
#define WGM1_CTC_OCR1A      (_BV(WGM12))
#define WGM1_FAST_PWM_ICR1  (_BV(WGM13) | _BV(WGM12) | _BV(WGM11))
#define WGM1_FAST_PWM_OCR1A (_BV(WGM13) | _BV(WGM12) | _BV(WGM11) | _BV(WGM10))

#define timer1_ov_enable()         { TIMSK |= _BV(TOIE1); }
#define timer1_ov_disable()        { TIMSK &= ~_BV(TOIE1); }
#define timer1_compa_enable()      { TIMSK |= _BV(OCIE1A); }
#define timer1_compa_disable()     { TIMSK &= ~_BV(OCIE1A); }
#define timer1_compb_enable()      { TIMSK |= _BV(OCIE1B); }
#define timer1_compb_disable()     { TIMSK &= ~_BV(OCIE1B); }
#define timer1_ic_enable()         { TIMSK |= _BV(TICIE1); }
#define timer1_ic_disable()        { TIMSK &= ~_BV(TICIE1); }
#define timer1_clear_coma_mode()   { TCCR1A &= ~(_BV(COM1A1) | _BV(COM1A0)); }
#define timer1_clear_comb_mode()   { TCCR1A &= ~(_BV(COM1B1) | _BV(COM1B0)); }

#define timer1_stop()\
	{ TCCR1B &= ~(_BV(CS12) | _BV(CS11) | _BV(CS10)); }

#define timer1_set_coma_mode(mode) {\
	timer1_clear_coma_mode();\
	TCCR1A |= (mode) & (_BV(COM1A1) | _BV(COM1A0));\
}

#define timer1_set_comb_mode(mode) {\
	timer1_clear_comb_mode();\
	TCCR1A |= (mode) & (_BV(COM1B1) | _BV(COM1B0));\
}

#define timer1_clear_wave_mode()   {\
	TCCR1A &= ~(_BV(WGM11) | _BV(WGM10));\
	TCCR1B &= ~(_BV(WGM13) | _BV(WGM12));\
}

#define timer1_set_wave_mode(mode) {\
	timer1_clear_wave_mode();\
	TCCR1A |= (mode) & (_BV(WGM11) | _BV(WGM10));\
	TCCR1B |= (mode) & (_BV(WGM13) | _BV(WGM12));\
}

#define timer1_start(clock) {\
	timer1_stop();\
	TCCR1B |= (clock) & (_BV(CS12) | _BV(CS11) | _BV(CS10));\
}

/*
 * TCNT1 ꤹ륫ͤꤹ롣
 *
 * 16ӥåȥ쥸˥뤿ᡢ16ӥåȥ쥸Ѥ
 * 쥸ؤ¾ʤ֤Ǽ¹Ԥ뤳ȡ
 */
static inline void 
timer1_set_count(uint16_t count)
{
	TCNT1 = count;
}


/*
 * OCR1A ꤹ륫ͤꤹ롣
 *
 * 16ӥåȥ쥸˥뤿ᡢ16ӥåȥ쥸Ѥ
 * 쥸ؤ¾ʤ֤Ǽ¹Ԥ뤳ȡ
 */
static inline void
timer1_set_ocr1a(uint16_t ocr1a)
{
	OCR1A = ocr1a;
}


/*
 * OCR1A ꤹ륫ͤꤹ롣
 *
 * 16ӥåȥ쥸˥뤿ᡢ16ӥåȥ쥸Ѥ
 * 쥸ؤ¾ʤ֤Ǽ¹Ԥ뤳ȡ
 */
static inline void
timer1_set_ocr1b(uint16_t ocr1b)
{
	OCR1B = ocr1b;
}


/*
 * ICR1 ꤹ륫ͤꤹ롣
 *
 * 16ӥåȥ쥸˥뤿ᡢ16ӥåȥ쥸Ѥ
 * 쥸ؤ¾ʤ֤Ǽ¹Ԥ뤳ȡ
 */
static inline void
timer1_set_icr1(uint16_t icr1)
{
	ICR1 = icr1;
}


void timer1_ov_set_interrupt(void (*func)(void));
void timer1_compa_set_interrupt(void (*func)(void));
void timer1_compb_set_interrupt(void (*func)(void));
void timer1_ic_set_interrupt(void (*func)(void));


#endif

/* End of timer1.h */
